<?php

namespace App\Http\Controllers;

use App\Helpers\Helper;
use App\Http\Requests\LoginRequest;
use App\Http\Requests\SocialLoginRequest;
use App\Http\Resources\UserResource;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Str;

class AuthenticationController extends Controller
{
    /**
     * Get a JWT via given credentials.
     *
     * @param LoginRequest $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function login(LoginRequest $request)
    {
        $credentials = $request->only(['user_name', 'password']);

        if (!auth()->attempt($credentials)) {
            return Helper::response([], 401, false, [__('user name or password is invalid')]);
        }

        $access_token = auth()->user()->createToken('API Token')->plainTextToken;
        auth()->user()->update(['login_at' => now()]);
        if ($request->has('udid')) {
            auth()->user()->userDevice($request, $access_token);
        }
        return Helper::response(["user" => new UserResource(auth()->user(), $access_token)], 200, false, [__('Logged in successfully')]);
    }

    /**
     *  socaial login with socia_id and provider
     *
     * @param  mixed $request
     * @return User
     */
    public function socialLogin(SocialLoginRequest $request)
    {
        $user = User::where([
            'social_id' => $request->social_id,
            'provider' => $request->provider
        ])
            ->first();
        // return $request;
        if (!$user) {
            $refrence_id = mt_rand(1000000000, 9999999999);
            $email = $request->provider . $refrence_id . '@bizzfeed.com';
            $user = User::create([
                'user_name' => $request->user_name,
                'email' => $request->email ?? $email,
                'social_id' => $request->social_id,
                'provider' => $request->provider,
                'password' => bcrypt(Str::random(16)),
                'user_type' => 'user',
            ]);
        }
        auth()->login($user);
        $access_token = auth()->user()->createToken('API Token')->plainTextToken;

        if ($request->has('udid')) {
            auth()->user()->userDevice($request, $access_token);
        }
        return Helper::response(['user' => new UserResource(auth()->user(), $access_token)], 200);
    }

    /**
     * guestLogin
     *
     * @param  mixed $request
     * @return User
     */


    public function guestLogin(Request $request)
    {
        $input = $request->all();
        do {
            $refrence_id = mt_rand(1000000000, 9999999999);
            $email = 'guset' . $refrence_id . '@bizzfeed.com';
        } while (User::where('email', $email)->exists());
        $user = User::create([
            'email' => $email,
            'password' => bcrypt(Str::random(16)),
            'user_type' => 'guest',
        ]);
        auth()->login($user);
        $access_token = auth()->user()->createToken('API Token')->plainTextToken;

        if ($request->has('udid')) {
            auth()->user()->userDevice($request, $access_token);
        }
        return Helper::response(['user' => new UserResource(auth()->user(), $access_token)], 200);
    }


    /**
     * Get logged in user
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function user()
    {
        return Helper::response(["user" => new UserResource(auth()->user(), auth()->login(auth()->user()))], 200);
    }


    /**
     * Log the user out (Invalidate the token).
     *
     * @return \Illuminate\Http\JsonResponse
     */

    public function logout(Request $request)
    {
        $user = $request->user();
        $user->update(['login_at' => null]);
        $user->currentAccessToken()->delete();
        if ($request->has('udid')) {
            $user->devices()->where('udid', $request->udid)->delete();
        }

        return Helper::response("", 200, false, [__('User has logged out')]);
    }
}
