<?php

namespace App\Http\Controllers;

use App\Helpers\Helper;
use App\Models\Incident;
use PHPUnit\TextUI\Help;
use App\Events\SendAction;
use Illuminate\Http\Request;
use App\Http\Resources\IncidentResource;
use App\Models\User;
use Illuminate\Support\Facades\Validator;
use function PHPUnit\Framework\returnSelf;

class IncidentController extends Controller
{


    public function add(Request $request)
    {
        $inputs = $this->validateInputs($request);
        $inputs['user_id'] = auth()->id();
        $inputs['date'] = date('Y-m-d', strtotime($inputs['date']));
        $inputs['company_id'] = auth()->user()->company_id;
        if ($request->hasFile('media')) {
            $inputs['media'] = $this->media1($request, 'incident');
        }
        if ($request->has('incident_id')) {
            $incident = Incident::findOrFail($request->incident_id);
            $incident->update($inputs);
            $response = $incident->fresh();
        } else {
            $incident = Incident::create($inputs);
            $response = $incident->fresh();
            $admin = User::where('company_id', auth()->user()->company_id)->where('user_role', 'sub_admin')->first();
            event(new SendAction([$admin->id], [
                'actor_id' => auth()->user()->id,
                'actor_type' => 'user',
                'user_id' => $admin->id,
                'object_id' => $incident->id,
                'title' => auth()->user()->user_name . '  upload incident',
                'object_type' => get_class($incident),
                'type' => 'create-incident',
                // 'media' => $media,
                // 'object_id' => '',
                'body' => $incident->description,
            ], true));
        }
        return Helper::response($response, 200);
    }

    function index()
    {
        $incidents = Incident::where('user_id', auth()->user()->id)->get();
        return Helper::response(IncidentResource::collection($incidents), 200);
    }

    function show($incident)
    {
        $incident = Incident::find($incident);
        return Helper::response($incident, 200);
    }


    // Private method for input validation
    private function validateInputs(Request $request)
    {
        return Validator::make($request->all(), [
            'name'        => 'sometimes|max:255',
            'phone'       => 'sometimes|max:15',
            'email'       => 'sometimes|email',
            'date'        => 'required|date',
            'time'        => 'required|max:10',
            'location'    => 'required|max:255',
            'description' => 'required|max:255',
            // 'media'       => 'required',
        ])->validate();
    }
}
