<?php

namespace App\Http\Controllers;

use App\Models\React;
use App\Helpers\Helper;
use App\Models\NewsFeed;
use App\Events\SendAction;
use Illuminate\Http\Request;
use App\Models\NewsFeedReachUser;
use Illuminate\Support\Facades\Auth;
use App\Http\Resources\NewsFeedResource;
use App\Models\Comment;
use GuzzleHttp\Promise\Create;
use Illuminate\Validation\Rule;

class NewsFeedController extends Controller
{
    public function likeDislike(Request $request)
    {
        $request->validate([
            'reactable_id' => 'required|integer',
            'reaction' => 'required|boolean'
        ]);
        $user = Auth::user();
        $news_feed = NewsFeed::find($request->reactable_id);
        if ($news_feed) {
            $reaction = $request->reaction;
            React::updateOrCreate(
                [
                    'user_id' => $user->id,
                    'reactable_type' => NewsFeed::class,
                    'reactable_id' => $news_feed->id,
                ],
                ['reaction' => $reaction]
            );
            if ($reaction == 1) {
                event(new SendAction([$news_feed->user_id], [
                    'actor_id' => $user->id,
                    'actor_type' => 'user',
                    'user_id' => $news_feed->user_id,
                    'object_id' => $news_feed->id,
                    'title' => 'Someone react your news feed',
                    'object_type' => get_class($news_feed),
                    'type' => 'reacte-new-feed',
                    // 'media' => $media,
                    // 'object_id' => '',
                    'body' => 'Someone react your news feed',
                ], true));
            }
            return Helper::response(['message' => 'Reaction recorded.'], 200);
        } else {
            return Helper::response([], 401, false, 'Reaction cannot be recorded at this time.');
        }
    }

    public function index()
    {
        $companyId = auth()->user()->company_id;
        $polls = NewsFeed::where('company_id', $companyId)->latest()->get();
        foreach ($polls as $poll) {
            NewsFeedReachUser::firstOrCreate([
                'news_feed_id' => $poll->id,
                'user_id' => auth()->id(),
            ]);
        }

        return Helper::response(NewsFeedResource::collection($polls), 200);
    }

    function comment(Request $request)
    {
        $validate = $request->validate([
            'news_feed_id' => [
                'required',
                Rule::exists('news_feeds', 'id'),
            ],
            'message' => 'required|string|max:100',
        ]);
        $validate['user_id'] = auth()->user()->id;
        $comment = Comment::create($validate);
        return Helper::response($comment, 200);
    }
}
