<?php

namespace App\Http\Controllers;

use App\Helpers\Helper;
use App\Http\Requests\ForgotPasswordRequest;
use App\Http\Requests\PasswordResetRequest;
use App\Http\Resources\UserResource;
use App\Jobs\ProcessVerification;
use App\Models\User;
use Illuminate\Auth\Events\PasswordReset;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Password;
use Symfony\Component\Console\Input\Input;

class PasswordResetController extends Controller
{
    // send code to email
    public function forgotPassword(ForgotPasswordRequest $request)
    {
        // return $request;

        $input = $request->all();
        if ($request->has('email')) {
            $user = User::where('email', $input['email'])->first();
            if ($user) {
                $user->forgot_password_code = 9999; //rand(1000, 9999);
                $user->save();
                Mail::send('mail.code', ['user' => $user], function ($m) use ($user) {
                    $m->from('donotreply@umpire-on-the-go.com') //from email
                        ->to($user->email) //to email
                        ->subject('Password reset pins'); //email subject
                });
                return Helper::response("", 200, false, ["Password reset pin sent to your email"]);
            } else {
                return Helper::response("", 404, false, ["User not found"]);
            }
        }

        if ($request->has('phone')) {
            $user = User::where('phone', $input['phone'])->first();
            if ($user) {
                $user->forgot_password_code = 9999; //rand(1000, 9999);
                $user->save();
                ProcessVerification::dispatch($user);
                return Helper::response("", 200, false, ["Password reset pin sent to phone"]);
            } else {
                return Helper::response("", 404, false, ["User not found"]);
            }
        }
    }

    public function verifyCode(Request $request)
    {
        // return ';;';
        $request->validate([
            'forgot_pasword_secret_key' => "required",
            'user_name' => "required"
        ]);
        $input = $request->all();
        if ($request->has('email')) {
            $user = User::where('email', $input['email'])->first();
        }
        if ($request->has('user_name')) {
            $user = User::where('user_name', $input['user_name'])->first();
        }

        if ($request->has('phone')) {
            $user = User::where('phone', $input['phone'])->first();
            ProcessVerification::dispatch($user);
        }

        if (!$user) {
            return Helper::response("", 404, false, ["User not found"]);
        }

        if ($user->forgot_pasword_secret_key != $input['forgot_pasword_secret_key']) {
            return Helper::response("", 404, false, ["secret key is not valid"]);
        }
        // if ($user->forgot_pasword_secret_key != $input['forgot_pasword_secret_key']) {
        //     return Helper::response("", 404, false, ["secret key is not valid"]);
        // }
        if (Auth::login($user)) {
            return Helper::response([], 401, false, [__('user name or password is invalid')]);
        }
        // return $user;
        $access_token = auth()->user()->createToken('API Token')->plainTextToken;
        $user->forgot_password_code = null;
        $user->save();
        return Helper::response(["user" => new UserResource(auth()->user(), $access_token)], 200);
    }

    /**
     * changePassword
     *
     * @param  mixed $request
     * @return void
     */
    public function changePassword(Request $request)
    {
        $request->validate(
            [
                'password' => 'required|confirmed||min:8|max:16|regex:/^(?=.*?[A-Z])(?=.*?[a-z])(?=.*?[0-9]).{6,}$/',
            ],
            [
                'password.confirmed' => "New password and confirm password does not match.",
                'password.regex' => "Include at least one upper case characters lower case character and a number",
                'password.min' => 'Password must be Minimum 8 Characters and Maximum 16 Characters.',
                'password.max' => 'Password must be Minimum 8 Characters and Maximum 16 Characters.'
            ]
        );
        $user = $user = Auth::user();
        // if (Hash::check($request->password, $user->password)) {
        //     return Helper::response("", 401, false, ["New password and Confirm password do not match"]);
        // }
        // return $request;
        $user->password = $request->password;
        $user->save();
        return Helper::response("", 200, false, ["Password has been reset successfully"]);
    }
    /**
     * changeUserPassword
     *
     * @param  mixed $request
     * @return void
     */
    public function changeUserPassword(Request $request)
    {
        $request->validate(
            [
                'current_password' => 'required|current_password',
                'password' => 'required|confirmed||min:8|regex:/^(?=.*?[A-Z])(?=.*?[a-z])(?=.*?[0-9]).{6,}$/|different:current_password',
            ],
            [
                'current_password.current_password' => "The current password is incorrect.",
                'password.confirmed' => "New password and confirm password does not match.",
                'password.regex' => "Include at least one upper case characters lower case character and a number"
            ]
        );
        $user = $user = Auth::user();
        $user->password = $request->password;
        $user->save();
        return Helper::response("", 200, false, ["Password changed successfully"]);
    }
}
