<?php

namespace App\Http\Controllers;

use App\Models\React;
use App\Helpers\Helper;
use App\Events\SendAction;
use App\Http\Resources\PollResource;
use App\Http\Resources\SuggestionResource;
use App\Models\Poll;
use App\Models\PollAnswer;
use App\Models\Suggestion;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
// use Illuminate\Support\Facades\Auth;

class ReactController extends Controller
{

    public function reactToPollOrSuggestion(Request $request)
    {
        $request->validate([
            'reactable_id' => 'required|integer',
            'type' => 'required|string|in:suggestion,poll',
            'reaction' => 'required|boolean'
        ], [
            'type.in' => 'The : Type attribute must be either suggestion or poll.',
        ]);
        $user = Auth::user();
        $object = null;
        $reactableId = null;
        $objectId = null;
        if ($request->type == 'poll') {
            $object = PollAnswer::find($request->reactable_id);
            $reactableId    = $object ? $object->id : null;
            $objectId       = $object ? $object->poll_id : null;
            $poll           = Poll::where('id', $objectId)->first();
            $admin = User::where('company_id', $user->company_id)->where('user_role', 'sub_admin')->first();
            $user_id    = $admin->id ?? 0;
            $reaction   = true;
            $return     = new PollResource($poll);
        } elseif ($request->type == 'suggestion') {
            $object = Suggestion::find($request->reactable_id);
            $reactableId = $object ? $object->id : null;
            $objectId = $object ? $object->id : null;
            $reaction = $request->reaction;
            $user_id = $object->user_id;
            $return     = new SuggestionResource($object);
        }
        if ($object && $reactableId) {
            if ($user->hasReactedToPollOrSuggestion($objectId, $request->type)) {
                return Helper::response([], 401, false, ['You have already reacted to this ' . $request->type]);
            }
            event(new SendAction([$user_id], [
                'actor_id' => auth()->user()->id,
                'actor_type' => 'user',
                'user_id' => $user_id,
                'object_id' => $reactableId,
                'title' => 'Someone react your ' . strtolower($request->type),
                'object_type' => get_class($object),
                'type' => 'reacte-' . $request->type,
                // 'media' => $media,
                // 'object_id' => '',
                'body' => 'Someone react your ' . strtolower($request->type),
            ], true));
            $react = new React();
            $react->user_id = $user->id;
            $react->reactable_type = get_class($object);
            $react->reactable_id = $reactableId;
            $react->reaction = $reaction;
            $react->save();
            return Helper::response($return, 200, false, ['message' => 'Reaction recorded.']);
        } else {
            return Helper::response([], 401, false, 'Reaction cannot be recorded at this time.');
        }
    }




    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        //
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        //
    }

    /**
     * Display the specified resource.
     */
    public function show(React $react)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(React $react)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, React $react)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(React $react)
    {
        //
    }
}
