<?php

namespace App\Http\Controllers;

use App\Helpers\Helper;
use App\Http\Requests\UserRegistrationRequest;
use App\Http\Resources\UserResource;
use App\Jobs\ProcessSignUp;
use App\Jobs\ProcessVerification;
use App\Models\User;
use Illuminate\Support\Str;
use Spatie\Activitylog\Contracts\Activity;
use Spatie\Activitylog\Models\Activity as ModelsActivity;

class RegistrationController extends Controller
{
    /**
     * User sign up
     *
     * @param RegistrationRequest $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function register(UserRegistrationRequest $request)
    {

        // return  $request;
        $parems = $this->attributes($request);
        if ($request->hasFile("profile_image")) {
            $parems["profile_image"] = $this->media3($request, "user");
        }
        $parems['forgot_pasword_secret_key'] = Str::random(60);
        ProcessSignUp::dispatch(
            $user = User::create($parems)
        );
        // return $user;
        $log = auth()->login($user);
        auth()->user()->update(['login_at' => now()]);
        $access_token = $user->createToken('API Token')->plainTextToken;

        if ($request->has('udid')) {
            auth()->user()->userDevice($request, $access_token);
        }
        //ProcessVerification::dispatch(auth()->user());
        return Helper::response(["user" => new UserResource($user->fresh(), $access_token)], 201, false, [__('Account created successfully')]);
    }

    /**
     * Sign up attributes
     *
     * @param $request
     * @return mixed
     */
    private function attributes($request)
    {
        return $request->only([
            'user_name',
            'company_id',
            'department_id',
            'profile_image',
            'gender',
            'user_type',
            'user_role',
            'phone',
            'password',
            'udid',
            'device_type',
            'device_token',
            'device_brand',
            'device_model',
            'device_os',
            'address',
        ]);
    }
}
