<?php

namespace App\Http\Controllers;

use Exception;
use Carbon\Carbon;
use App\Helpers\Helper;
use App\Models\Package;
use App\Models\Subscription;
use Illuminate\Http\Request;

class SubscriptionController extends Controller
{

    /**
     * Create stripe customer
     *
     * @param Request $request
     * @return JsonResponse
     */
    public function subscribe(Request $request)
    {
        $request->validate([
            "package_slug" => "required",
            "charge_id"    => "required",
            "currency"    => "required"
        ]);
        // if (empty(Helper::getActiveSubscription())) {
        //     $customer = $this->createStripeCustomer();

        //     try {
        //         $this->createStripeCard($customer, $request->token);
        //     } catch (\Exception $e) {
        //         return Helper::responseHelper([], 406, [$e->getMessage()]);
        //     }

        //     PackageSubscribed::dispatch(
        //         $charge = $this->charge($customer, $request->payment_amount), Package::find($request->package_id)
        //     );

        //     return Helper::responseHelper(auth()->user()->fresh(), 200);
        // }
        try {
            $input = $request->all();

            $input['user_id'] = $request->user()->id;
            $input['status'] = 'active';
            $package = Package::where('slug', $input['package_slug'])->first();
            if ($package == null) {
                return Helper::response('', 404, false, ['Package not found']);
            }
            $input['package_id'] = $package->id;
            $input['amount'] = $package->total_amount;
            $input['start_date'] = Carbon::now()->format('Y-m-d H:i:s');
            $days = $package->number_of_months * 30;
            $input['end_date']  = Carbon::now()->add($days, 'day')->format('Y-m-d H:i:s');
            // return $input;
            $subscription = auth()->user()->subscriptions()->create($input);

            $subscription->charge($request->charge_id, url('invoice/callback'));
            if ($subscription->redirect_url != null) {
                return Helper::response(['advertisment' => $subscription], 200, false, 'Payment action required');
            }

            return Helper::response(['subscription' => $subscription], 200, false, ['Subscribed Successfully']);
        } catch (Exception $e) {
            return Helper::response(null, 422, null, [$e->getMessage()]);
        }
    }

    public function getSubscription(Request $request)
    {
        $subs = auth()->user()->subscriptions()->latest()->first();
        return Helper::response(['subscription' => $subs], 200, false, []);
    }
}
