<?php

namespace App\Http\Controllers;

use App\Models\User;
use App\Helpers\Helper;
use App\Models\Childern;
use Illuminate\Http\Request;
use App\Jobs\ProcessVerification;
use App\Http\Resources\UserResource;
use App\Http\Requests\UserUpdateRequest;
use App\Http\Requests\ProfileUpdateRequest;
use App\Models\Product;

class UserController extends Controller
{

    /**
     * createOrUpdateProfile
     *
     * @param  mixed $request
     * @return void
     */
    public function createOrUpdateProfile(UserUpdateRequest $request)
    {
        $is_available = User::where('id', '!=', auth()->user()->id)->where('user_name', $request['user_name'])->first();
        if ($is_available) {
            return Helper::response([], 401, false, [__('The name has already been taken.')]);
        }
        $attributes = $request->all();
        if (auth()->user()->user_type == 'user') {
            if ($request->hasFile("profile_image")) {
                $attributes["profile_image"] = $this->media2($request, "user");
            }
            // if ($request->hasFile('images')) {
            //     $this->uploadMedia($request, auth()->user());
            // }
            // if ($request->has('vehicle_details')) {

            //     auth()->user()->userVehicle()->updateOrCreate([
            //         'vehicle_type' => $attributes['vehicle_type'],
            //         'vehicle_make' => $attributes['vehicle_make'],
            //         'vehicle_model' => $attributes['vehicle_model'],
            //         'vehicle_year' => $attributes['vehicle_year'],
            //         'vehicle_color' => $attributes['vehicle_color'],
            //         'vehicle_plate_number' => $attributes['vehicle_plate_number'],
            //     ]);
            // }
        }
        $user = tap(User::find(auth()->user()->id))->update($attributes);
        //$user1 = User::with('userMedia', 'userVehicle')->find(auth()->user()->id);



        return Helper::response(["user" => new UserResource($user, $request->bearerToken())], 200, false, [__('Profile updated successfully')]);
    }

    /**
     * Find user by id
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function getUserDetails($id)
    {
        $user = User::find(Helper::decode($id));

        return Helper::response(["user" => new UserResource($user)], 200);
    }
    /**
     * get auth user profile
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function getAuthUserDetails()
    {
        // $user = User::find(auth()->user()->id);
        $user = User::find(auth()->user()->id);
        // $myChildern_reports = Childern::with('myReports')->where('user_id', auth()->user()->id)->take(2)->get();
        // return Helper::response(['myChildern_reports' => $myChildern_reports], 200, false);
        return Helper::response(["user" => new UserResource($user)], 200);
    }

    public function deleteAccount()
    {
        $user = User::find(auth()->user()->id);
        $user->delete();
        return Helper::response([], 200, [__('Your account has been deleted successfully')]);
    }


    public function updatePicture(Request $request)
    {

        $parame = $this->attributes($request);
        if ($request->hasFile('profile_image')) {
            $parame['profile_image'] = $this->media2($request, "user");
        }
        User::where('id', auth()->user()->id)->update($parame);
        $user = auth()->user()->fresh();
        // return Helper::response(["user" => new UserResource($user)], 200, false, [__('Your liscence has been updated successfully')]);
        return Helper::response(["user" => new UserResource($user)], 200, false);
    }

    private function uploadMedia($request, $user)
    {
        if ($request->images !== null) {

            foreach ($request->images as $key => $medium) {
                if (is_file($medium)) {
                    $user->userMedia()->where('media_title', $key)->delete();

                    $file = \Storage::disk(env('DISK'))->put('user', $medium);

                    $user->userMedia()->create([
                        'media_type' => $medium->getClientMimeType(),
                        'media_url' => $file,
                        'media_title' => $key
                    ]);
                }
            }
        }
    }

    public function addDevice(Request $request)
    {
        if ($request->has('udid')) {
            auth()->user()->userDevice($request, auth()->user()->access_token);
        }
        return Helper::response([], 200, null, [__('Device added successfully')]);
    }

    public function deleteDevice(Request $request)
    {
        if ($request->has('udid')) {
            auth()->user()->devices()->where('udid', $request->udid)->delete();
        }
        return Helper::response([], 200, null, [__('Device deleted successfully')]);
    }




    private function attributes($request)
    {
        return $request->only([
            'first_name',
            'last_name',
            'profile_image',
            'full_name',
            'phone',
            'gender',
            'dob',
            'user_type',
        ]);
    }
}
