<?php

namespace App\Http\Resources;

use App\Helpers\Helper;
use App\Models\Department;
use Illuminate\Http\Resources\Json\JsonResource;

class UserResource extends JsonResource
{
    /**
     * @var mixed|null $accessToken
     */
    private $accessToken;

    /**
     * UserResource constructor.
     * @param $resource
     * @param null $accessToken
     */
    public function __construct($resource, $accessToken = null)
    {
        parent::__construct($resource);
        $this->accessToken = $accessToken;
        $this->resource = $resource;
    }

    /**
     * Transform the resource into an array.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return array|\Illuminate\Contracts\Support\Arrayable|\JsonSerializable
     */
    public function toArray($request)
    {
        $department = Department::where('id', $this->department_id)->first();
        $response = parent::toArray($request);

        $response['id'] = Helper::encode($this->id);
        $response['token'] = $this->accessToken;
        $response['permissions'] = $this->permissions;
        $response['department'] = $department->name ?? null;
        $response['token_type'] = 'bearer';
        $response['profile_image'] = $this->profile_image !== null ? env('MEDIA_URL') . $this->profile_image : null;
        $response['issue_date'] = $this->issue_date !== null ? $this->issue_date : null;
        unset($response['deleted_at']);

        return $response;
    }
}
