<?php

namespace App\Models;

// use Stripe\Review;
use App\Models\Review;
use App\Models\Category;
use App\Models\ProductImage;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Factories\HasFactory;

class Product extends Model
{
    use HasFactory;
    protected $fillable = [
        'id',
        'category_id',
        'sub_category_id',
        'brand',
        'name',
        'description',
        'long_description',
        'featured',
        'status',
    ];

    protected $appends = ['avg_rating', 'favorite'];

    public function getAvgRatingAttribute()
    {
        return $this->reviews->avg('rating');
    }

    public function getFavoriteAttribute()
    {
        return $this->favourite->isNotEmpty();
    }



    /**
     * Get the caregory that owns the Product
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     */
    public function category()
    {
        return $this->belongsTo(Category::class, 'category_id');
    }
    /**
     * Get the caregory that owns the Product
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     */
    public function subCaregory()
    {
        return $this->belongsTo(Category::class, 'sub_category_id');
    }

    /**
     * Get all of the comments for the Product
     *
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function productAttribute()
    {
        return $this->hasMany(ProductAttribute::class,);
    }
    public function attributes()
    {
        return $this->hasMany(ProductAttribute::class,);
    }
    public function productImage()
    {
        return $this->hasMany(ProductImage::class,);
    }
    public function orderDetails()
    {
        return $this->hasMany(OrderDetail::class, 'product_id');
    }
    public function reviews()
    {
        return $this->morphMany(Review::class, 'recordable');
    }
    public function favourite()
    {
        return $this->morphMany(Favourite::class, 'favorable');
    }
}
