<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\Auth;

class Suggestion extends Model
{
    use HasFactory;

    protected $fillable = [
        'user_id',
        'company_id',
        'description',
        'media_type',
        'media',
        'reason',
        'status',
        'name',
        'email',
    ];

    protected $appends = [
        'isLike',
        'isDislike',
    ];

    public function likes()
    {
        return $this->morphMany(React::class, 'reactable')->where('reaction', true);
    }

    public function dislikes()
    {
        return $this->morphMany(React::class, 'reactable')->where('reaction', false);
    }

    public function user()
    {
        return $this->belongsTo(User::class, 'user_id')->select('id', 'first_name', 'last_name', 'user_name', 'profile_image');
    }

    public function getIsLikeAttribute()
    {
        return $this->likes->contains('user_id', Auth::id());
    }

    public function getIsDislikeAttribute()
    {
        return $this->dislikes->contains('user_id', Auth::id());
    }
}
