<?php

namespace App\Models;

// use Illuminate\Contracts\Auth\MustVerifyEmail;

use Carbon\Carbon;
use App\Models\React;
use App\Helpers\Helper;
use Illuminate\Support\Str;
use Laravel\Sanctum\HasApiTokens;
use Spatie\Permission\Models\Role;
use Illuminate\Notifications\Notifiable;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Foundation\Auth\User as Authenticatable;

class User extends Authenticatable
{
    use HasApiTokens, HasFactory, Notifiable;

    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    protected $fillable = [

        'department_id',
        'company_id',
        'user_name',
        // 'forgot_password_code',
        'email',
        'login_at',
        'social_id',
        'provider',
        'phone',
        'password',
        'is_notify',
        'gender',
        'dob',
        'profile_image',
        'customer_stripe_id',
        'forgot_pasword_secret_key',
        'user_type',
        'user_role',
    ];

    /**
     * The attributes that should be hidden for serialization.
     *
     * @var array<int, string>
     */
    protected $hidden = [
        'password',
        'remember_token',
    ];

    /**
     * The attributes that should be cast.
     *
     * @var array<string, string>
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
    ];

    // check subscriptions
    protected $appends = [
        'is_subscribed',
        'company_logo',
        'permissions',
    ];
    public function getIsSubscribedAttribute()
    {
        $subscription = $this->subscriptions()->latest()->first();
        // return Carbon::now()->format('Y-m-d');
        if ($subscription) {
            if ($subscription->end_date >= Carbon::now()->format('Y-m-d')) {
                return true;
            }
        }
        return false;
    }
    public function getCompanyLogoAttribute()
    {

        $profileImage = User::where('user_role', 'sub_admin')
            ->where('company_id', auth()->user()->company_id)
            ->value('profile_image');
        return $profileImage ?: null;
    }
    public function getPermissionsAttribute()
    {
        // Retrieve the admin (sub_admin) based on the authenticated user's company_id
        $admin = User::where('company_id', auth()->user()->company_id)
            ->where('user_role', 'sub_admin')
            ->first();

        // Check if the admin (sub_admin) exists
        if ($admin) {
            // Get permissions of the admin (sub_admin)
            $adminPermissions = $admin->rolesCustomRelation()->with('permissions1')->get()
                ->flatMap(function ($role) {
                    return $role->permissions1->pluck('name');
                });

            return $adminPermissions;
        }

        // If no admin (sub_admin) found, return an empty collection
        return collect();
    }
    public function rolesCustomRelation()
    {
        return $this->belongsToMany(Role::class, 'model_has_roles', 'model_id', 'role_id')->limit(1);
    }

    /**
     * The booted method of the model
     */
    protected static function booted()
    {
        static::creating(function ($user) {
            $user->verification_code = mt_rand(10000, 99999);
            $user->phone_verification_code = mt_rand(10000, 99999);
            $user->expired_at = Carbon::now()->addMinutes(60);
        });
    }


    public function verifyUser()
    {
        $this->phone_verified_at = now();
        $this->phone_verification_code = NULL;
        $this->save();
    }

    /**
     * Set password encryption when insert
     *
     * @param $password
     */
    public function setPasswordAttribute($password)
    {
        $this->attributes['password'] = bcrypt($password);
    }

    /**
     * Get formatted timestamp of email_verified_at
     *
     * @param $date
     * @return string
     */
    public function getExpiredAtAttribute($date)
    {
        if ($date !== null) {
            return Carbon::parse($date)->format('Y-m-d H:i:s');
        }
    }

    /**
     * Mark user verified
     */
    public function markUserVerified()
    {
        $this->verification_code = null;
        $this->expired_at = null;
        $this->email_verified_at = now();

        $this->save();
    }

    public function hasReactedToPollOrSuggestion($Id, $type)
    {
        if ($type == 'poll') {
            $answerids = PollAnswer::where('poll_id', $Id)->pluck('id')->toArray();
            // $react = React::where('reactable_type', PollAnswer::class)->whereIn(['reactable_id' => $answerids, 'user_id' => auth()->user()->id])->count();
            $react = React::where('reactable_type', PollAnswer::class)
                ->whereIn('reactable_id', $answerids)
                ->where('user_id', auth()->user()->id)
                ->count();
            if ($react != 0) {
                return true;
            } else {
                return false;
            }
            // return $this->reacts()
            //     ->whereHasMorph('reactable', 'App\Models\PollAnswer', function ($query) use ($Id, $answerids) {
            //         $query->whereIn('reactable_id', $answerids);
            //     })
            //     ->exists();
        } elseif ($type == 'suggestion') {
            // dd($type);
            return $this->reacts()
                ->whereHasMorph('reactable', 'App\Models\Suggestion', function ($query) use ($Id) {
                    $query->where('id', $Id);
                })
                ->exists();
        } else {
            return false;
        }
    }

    public function reacts()
    {
        return $this->hasMany(React::class);
    }


    /**
     * Check if verification code is not expired
     *
     * @return bool
     */
    public function hasValidCode()
    {
        return $this->verification_code !== null && Carbon::parse($this->expired_at)->addMinutes(60) > now();
    }

    public function card()
    {
        return $this->hasMany(UserCard::class);
    }
    public function devices()
    {
        return $this->hasMany(UserDevice::class);
    }

    public function helpAndSupport()
    {
        return $this->hasMany(HelpAndSupport::class);
    }
    public function childern()
    {
        return $this->hasMany(Childern::class);
    }



    public function userDevice($request, $access_token)
    {
        $request['access_token'] = $access_token;
        $this->devices()->updateOrCreate(
            $request->only('udid'),
            $request->all()
        );
    }

    public function subscriptions()
    {
        return $this->hasMany(Subscription::class);
    }

    public function favorites()
    {
        return $this->hasMany(Favourite::class, 'user_id');
    }
}
